/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package quicksync;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.ServerSocket;
import java.net.Socket;
import javafx.application.Platform;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.TextField;
import javafx.scene.input.KeyEvent;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.AnchorPane;
import javafx.scene.layout.Pane;
import javafx.scene.media.Media;
import javafx.scene.media.MediaPlayer;
import javafx.scene.media.MediaView;
import javafx.scene.paint.Color;
import javafx.scene.shape.Circle;
import javafx.stage.FileChooser;
import javafx.stage.FileChooser.ExtensionFilter;
import javafx.stage.Stage;
import javafx.stage.StageStyle;

/**
 *
 * @author admin
 */
public class FXMLMainController extends AnchorPane {

    @FXML
    AnchorPane uiLayer;
    @FXML
    AnchorPane mainPane;
    @FXML
    Button load;
    @FXML
    Button reset;
    @FXML
    Button playPause;
    @FXML
    Circle indicator;
    @FXML
    Label offsetInd;
    @FXML
    TextField portField;
    @FXML
    Pane dragBar;

    int currentFrame = 0;

    boolean playing = false;

    MediaView mView = new MediaView();

    ShortServ server = new ShortServ();

    double totalDuration = 1;

    int frameOffset = QuickSync.frameOffset;

    boolean guiOn = false;

    double initialX, initialY = 0.0;

    Stage stage;

    //MidiHandler cntrl = new MidiHandler();
    public FXMLMainController() {

        FXMLLoader fxmlLoader = new FXMLLoader(getClass().getResource("FXMLMain.fxml"));
        fxmlLoader.setRoot(this);
        fxmlLoader.setController(this);

        try {
            fxmlLoader.load();
        } catch (IOException exception) {
            throw new RuntimeException(exception);
        }

        mainPane.getChildren().add(0, mView);
        AnchorPane.setBottomAnchor(mView, 1.0);
        AnchorPane.setTopAnchor(mView, 1.0);
        AnchorPane.setLeftAnchor(mView, 1.0);
        AnchorPane.setRightAnchor(mView, 1.0);

        server.port = Integer.parseInt(portField.getText());

        mainPane.setOnKeyTyped((KeyEvent ke) -> {
            handleKeyPressed(ke);
        });

        if (QuickSync.autoLoadFile) {
            handleLoad(new ActionEvent());
        }
        if (QuickSync.autoLaunchServer) {
            portField.setText("" + QuickSync.serverPort);
            server = new ShortServ(QuickSync.serverPort);
            server.start();
        }

        dragBar.setOnMousePressed((MouseEvent e) -> {
            initialX = e.getSceneX();
            initialY = e.getSceneY();
        });

        dragBar.setOnMouseDragged((MouseEvent e) -> {
            getScene().getWindow().setX(e.getScreenX() - initialX);
            getScene().getWindow().setY(e.getScreenY() - initialY);
            stage.setFullScreen(false);
        });

        Platform.runLater(() -> {
            stage = (Stage) getScene().getWindow();
        });

        updateOffsetLabel();
    }

    public void handleKeyPressed(KeyEvent ke) {
        char kode = ke.getCharacter().charAt(0);
        System.out.println("key");
        switch (kode) {
            case 'a':
                handleIncrFrameOffset(new ActionEvent());
                break;
            case 'b':
                handleDecrFrameOffset(new ActionEvent());
                break;
            case 'c':
                handlePlayPause(new ActionEvent());
                break;
            case 'r':
                handleReset(new ActionEvent());
                break;

        }

    }

    public int getCurrentFrame() {
        double result = 0;
        if (mView != null && mView.getMediaPlayer() != null) {
            result = mView.getMediaPlayer().getCurrentTime().toMillis() / (30.0);
            //System.out.println(result);
        }
        return (int) result;
    }

    @FXML
    public void handleFullScreen(ActionEvent e) {
        // do what you have to do
        stage.setFullScreen(!stage.isFullScreen());
    }

    @FXML
    public void handleClose(ActionEvent e) {
        // do what you have to do
        stage.close();
    }

    @FXML
    public void handleGUIHide(ActionEvent e) {
        guiOn = !guiOn;
        uiLayer.setVisible(guiOn);

    }

    @FXML
    public void handleServerReset(ActionEvent e) {
        if (server != null) {
            try {
                Socket c = new Socket("localhost", server.port);
                PrintWriter out = new PrintWriter(c.getOutputStream(), true);
                BufferedReader in = new BufferedReader(new InputStreamReader(c.getInputStream(), "UTF-8"));
                out.println("4,0");
                in.readLine();
                server.terminate();
                out.println("rektg");
                //in.readLine();

            } catch (IOException ex) {

            }
        }
        server = new ShortServ(Integer.parseInt(portField.getText()));
        server.start();
    }

    @FXML
    public void handleIncrFrameOffset(ActionEvent e) {
        frameOffset += 1;
        updateOffsetLabel();
    }

    @FXML
    public void handleDecrFrameOffset(ActionEvent e) {
        frameOffset -= 1;
        updateOffsetLabel();
    }

    public void updateOffsetLabel() {
        offsetInd.setText("" + frameOffset);
    }

    @FXML
    public void handlePlayPause(ActionEvent e) {
        playing = !playing;

        if (playing) {
            playPause.setText("Pause");
            try {
                //mView.getMediaPlayer().play();
                server.currCmd = 'p';
            } catch (Exception ex) {
                System.out.println("no file or something");
                handleReset(e);
            }
        } else {
            playPause.setText("Play");
            try {
                mView.getMediaPlayer().pause();
                server.currCmd = 'q';
            } catch (Exception ex) {
                System.out.println("no file or something");
            }
        }

    }

    @FXML
    public void handleReset(ActionEvent e) {
        try {
            mView.getMediaPlayer().stop();
            currentFrame = 0;
            server.currCmd = 'r';
        } catch (Exception ex) {
            System.out.println("no file or something");
        }
        playing = false;
        playPause.setText("Play");
    }

    @FXML
    public void handleLoad(ActionEvent e) {

        mView.setMediaPlayer(null);

        FileChooser fChooser = new FileChooser();
        fChooser.setTitle("Open Media File");
        fChooser.getExtensionFilters().addAll(new ExtensionFilter("Video Files", "*.mp4"));
        try {
            File f;
            if (QuickSync.autoLoadFile) {
                f = new File(QuickSync.filePath);
            } else {
                f = fChooser.showOpenDialog(getScene().getWindow());
            }
            Media m = new Media(f.toURI().toString());

            MediaPlayer mPlayer = new MediaPlayer(m);
            mView.setMediaPlayer(mPlayer);

            mPlayer.currentTimeProperty().addListener((observableValue, oldTime, newTime) -> {
//                int frame = (int) (newTime.toMillis() / 30.0);
//                if (frame % 10 == 0) {
//                    System.out.println("time: " + frame);
//                }

            });

            mPlayer.setOnReady(() -> {
                totalDuration = mPlayer.getMedia().getDuration().toMillis();
                System.out.println("total duration: " + totalDuration);
                System.out.println("height: " + mPlayer.getMedia().getHeight() + "\nwidth: " + mPlayer.getMedia().getWidth());
                mainPane.setPrefHeight(mPlayer.getMedia().getHeight());
                mainPane.setPrefWidth(mPlayer.getMedia().getWidth());
                double x = 41;
                double y = 18;
                if (stage.getStyle() == StageStyle.UNDECORATED) {
                    x = 0;
                    y = 0;
                }
                getScene().getWindow().setHeight(mPlayer.getMedia().getHeight() + x);
                getScene().getWindow().setWidth(mPlayer.getMedia().getWidth() + x);
            });

        } catch (Exception ex) {
            System.out.println("well it looks like you messed up: " + ex);
        }

    }

    private class ShortServ extends Thread {

        private boolean terminate = false;
        ServerSocket listener;
        Socket client;
        int port = 5515;
        BufferedReader in;
        PrintWriter out;

        char currCmd = '.';

        public ShortServ(int port) {
            this.port = port;
            this.setDaemon(true);
        }

        public ShortServ() {
            this(5515);
        }

        public boolean terminate() {
            terminate = true;
            interrupt();
            return true;
        }

        @Override
        public void interrupt() {
            super.interrupt();
        }

        @Override
        public void run() {
            try {
                listener = new ServerSocket(port);

                System.out.println("The server is running on port: " + port);
                Platform.runLater(() -> {
                    indicator.setFill(Color.web("FF0000", 0.8));
                });

                client = listener.accept();

                BufferedReader in = new BufferedReader(new InputStreamReader(client.getInputStream(), "UTF-8"));
                PrintWriter out = new PrintWriter(client.getOutputStream(), true);

                // Send a welcome message to the client.
                String input = in.readLine();
                int robotFrames = Integer.parseInt(input.split(",")[1]);
                System.out.println("robotFrames: " + robotFrames);
                out.println("server accepted");
                Platform.runLater(() -> {
                    indicator.setFill(Color.web("FF8800", 0.8));
                });

                while (!terminate && currCmd != ';') {
                    String str = in.readLine();
                    double fac = 0.0;
                    try {
                        fac = (mView.getMediaPlayer().getCurrentTime().toMillis() / totalDuration) * 100000.0;
                    } catch (Exception ex) {
                        System.out.println("no media loaded");
                    }
                    String numPart = str.split("\\|")[1];
                    String robState = str.split("\\|")[2];
                    //System.out.println("reported: " + numPart + ",actual: " + (fac * robotFrames) / 100000.0 + ",robState: " + robState);
                    if ((fac * robotFrames) / 100000.0 == robotFrames) {
                        currCmd = 'q';
                    }
                    int reportedFrame = Integer.parseInt(numPart);
                    out.println(currCmd + "," + fac + "," + frameOffset + "|");
                    if (currCmd == 'p') {
                        mView.getMediaPlayer().play();
                    }
                    if (robState.equals("s")) {
                        Platform.runLater(() -> {
                            indicator.setFill(Color.web("FFFF00", 0.8));
                        });
                    } else if (robState.equals("p")) {
                        Platform.runLater(() -> {
                            indicator.setFill(Color.web("00FF00", 0.8));
                        });
                    }
                }

                in.readLine();
                out.print(";");
                client.close();
                listener.close();

            } catch (IOException e) {
                System.out.println(e);
            } finally {
                System.out.println("server terminated");
            }

        }

    }
}
